@@ -2,7 +2,7 @@
 /*
  * Kanatest
  *
- * Copyright (C) 2001-2004, 2006 Tomasz Mąka <pasp@users.sourceforge.net>
+ * Copyright (C) 2001-2004, 2006 Tomasz Maka <pasp@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -64,6 +64,7 @@ GtkWidget   *label_le;
 GtkWidget   *romaji_entry;
 GtkWidget   *stop_button;
 GtkProgressBar *progressbar;
+GtkWidget   *hseparator_up;
 
 GdkPixbuf   *icon, *logo;
 
@@ -75,7 +76,7 @@ gboolean    test_state = FALSE;
 
 /*--------------------------------------------------------------------*/
 
-GtkWidget* 
+GtkWidget*
 gui_stock_label_button (gchar *blabel, const gchar *bstock) {
 
     GtkWidget   *button;
@@ -90,13 +91,15 @@ gui_stock_label_button (gchar *blabel, const gchar *bstock) {
     gtk_container_add (GTK_CONTAINER (alignment), hbox);
 
     image = gtk_image_new_from_stock (bstock, GTK_ICON_SIZE_BUTTON);
-    if (image)
-        gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, TRUE, 0);
 
-    if (blabel)
-    gtk_box_pack_start (GTK_BOX (hbox),
-        g_object_new (GTK_TYPE_LABEL, "label", blabel, "use_underline", TRUE, NULL),
-        FALSE, TRUE, 0);
+    if (image) {
+        gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, TRUE, 0);
+    }
+
+    if (blabel) {
+        gtk_box_pack_start (GTK_BOX (hbox),
+            g_object_new (GTK_TYPE_LABEL, "label", blabel, "use_underline", TRUE, NULL), FALSE, TRUE, 0);
+    }
 
     gtk_widget_show_all (alignment);
     gtk_container_add (GTK_CONTAINER (button), alignment);
@@ -106,134 +109,138 @@ gui_stock_label_button (gchar *blabel, const gchar *bstock) {
 
 /*--------------------------------------------------------------------*/
 
-void 
+void
 gui_set_progress (void) {
 
     gchar buffer[BUFFER_SIZE];
 
-    sprintf (buffer, "[ %d / %d ]", question_counter, max_entries_in_test);
-    gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar), buffer);
-    gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar), (gfloat)question_counter / max_entries_in_test); 
+    if (config.repeat_mode == REPEAT_ALL) {
+        gtk_widget_hide(GTK_WIDGET(progressbar));
+        gtk_widget_hide(GTK_WIDGET(hseparator_up));
+    } else {
+        gtk_widget_show(GTK_WIDGET(progressbar));
+        gtk_widget_show(GTK_WIDGET(hseparator_up));
+        sprintf (buffer, "[ %d / %d ]", question_counter, max_entries_in_test);
+        gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar), buffer);
+        gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar), (gfloat)question_counter / max_entries_in_test); 
+    }
     gtk_entry_set_text (GTK_ENTRY(romaji_entry), "");
 }
 
 /*--------------------------------------------------------------------*/
 
-void 
+void
 gui_buttons_handler_cb (GtkWidget *widget, gpointer data) {
 
     switch((gint)data) {
 
-    case B_STATS:   /* STATS */
-            stats_create_window ();
-            break;  
+        case B_STATS:   /* STATS */
+                stats_create_window ();
+                break;
 
-    case B_ABOUT:   /* ABOUT */
-            about_create_window ();
-            break;  
+        case B_ABOUT:   /* ABOUT */
+                about_create_window ();
+                break;
 
-    case B_OPTIONS: /* OPTIONS */
-            options_create_window ();
-            break;  
+        case B_OPTIONS: /* OPTIONS */
+                options_create_window ();
+                break;
 
-    case B_CHART:   /* CHART */
-            chart_create_window ();
-            break;  
+        case B_CHART:   /* CHART */
+                chart_create_window ();
+                break;
 
-    case B_STOP:    /* STOP */
-            test_state = FALSE;
-            gui_disable_test ();
-            break;  
+        case B_STOP:    /* STOP */
+                if (config.repeat_mode == REPEAT_ALL) {
+                    test_info();
+                } else {
+                    gui_disable_test();
+                }
+                break;
 
-    case B_START:   /* START */
+        case B_START:   /* START */
 
-            test_state = TRUE;
-            gtk_widget_show(char_label);
-            gtk_widget_hide(logo_area);
+                test_state = TRUE;
+                gtk_widget_show(char_label);
+                gtk_widget_hide(logo_area);
 
-            gui_set_widgets_status (FALSE);
+                gui_set_widgets_status (FALSE);
 
-            gtk_widget_grab_focus (romaji_entry);
+                gtk_widget_grab_focus (romaji_entry);
 
-            test_init();
-            test_generate_tables (max_entries_in_test);
+                test_init();
+                test_generate_tables (max_entries_in_test);
 
-            gui_display_kana (questions_table[question_counter], config.kana_mode);
-            gui_set_progress ();
+                gui_display_kana (questions_table[question_counter], config.kana_mode);
+                gui_set_progress ();
 
-            start_time = time (NULL);
-            break;  
+                start_time = time (NULL);
+                break;
 
-    default:
-            break;
+        default:
+                break;
     }
 }
 
 /*--------------------------------------------------------------------*/
 
-void 
+void
 gui_disable_test (void) {
 
+    test_state = FALSE;
+
     gtk_widget_show(logo_area);
     gtk_widget_hide(char_label);
 
     gui_set_widgets_status (TRUE);
 
-    gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar),
-                                _("Select test mode, kana set and press START"));
+    gtk_widget_show(GTK_WIDGET(progressbar));
+    gtk_widget_show(GTK_WIDGET(hseparator_up));
     gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar), 0.0);
     gtk_entry_set_text (GTK_ENTRY(romaji_entry), "");
 
-    gtk_image_set_from_pixbuf (GTK_IMAGE (logo_area), logo);
     gtk_widget_grab_focus (start_button);
+
+    while (g_main_context_iteration(NULL, FALSE));
+
+    gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar),
+                                _("Select test mode, kana set and press START"));
 }
 
 /*--------------------------------------------------------------------*/
 
-void 
-gui_set_widgets_status (gint mode) {
+void
+gui_set_widgets_status (gboolean mode) {
 
-    gboolean st_a, st_b;
-
-    st_a = TRUE;
-    st_b = FALSE;
-
-    if(mode) {
-        st_a = FALSE;
-        st_b = TRUE;
-    }       
-
-    gtk_widget_set_sensitive (romaji_entry, st_a);
-    gtk_widget_set_sensitive (frame_ro, st_a);
-    gtk_widget_set_sensitive (stop_button, st_a);
-
-    gtk_widget_set_sensitive (start_button, st_b);
-    gtk_widget_set_sensitive (stat_button, st_b);
-    gtk_widget_set_sensitive (quit_button, st_b);
-    gtk_widget_set_sensitive (about_button, st_b);
-    gtk_widget_set_sensitive (prefs_button, st_b);
-    gtk_widget_set_sensitive (chart_button, st_b);
-    gtk_widget_set_sensitive (combobox_kana_mode, st_b);
-    gtk_widget_set_sensitive (combobox_lesson, st_b);
-    gtk_widget_set_sensitive (label_ka, st_b);
-    gtk_widget_set_sensitive (label_le, st_b);
+    gtk_widget_set_sensitive (romaji_entry, !mode);
+    gtk_widget_set_sensitive (frame_ro, !mode);
+    gtk_widget_set_sensitive (stop_button, !mode);
 
+    gtk_widget_set_sensitive (start_button, mode);
+    gtk_widget_set_sensitive (stat_button, mode);
+    gtk_widget_set_sensitive (quit_button, mode);
+    gtk_widget_set_sensitive (about_button, mode);
+    gtk_widget_set_sensitive (prefs_button, mode);
+    gtk_widget_set_sensitive (chart_button, mode);
+    gtk_widget_set_sensitive (combobox_kana_mode, mode);
+    gtk_widget_set_sensitive (combobox_lesson, mode);
+    gtk_widget_set_sensitive (label_ka, mode);
+    gtk_widget_set_sensitive (label_le, mode);
 }
 
 /*--------------------------------------------------------------------*/
 
-void 
+void
 gui_display_kana (gint number, gint mode) {
 
     gchar tmpbuf[BUFFER_SIZE], letbuf[BUFFER_SIZE];
 
-
     switch (mode) {
         case HIRAGANA:
             if (number >= MIXED_SEPARATOR) {
                 number -= MIXED_SEPARATOR;
             }
-            sprintf (tmpbuf, "<span font_desc='80' face='%s' color='%s'>%s</span>", 
+            sprintf (tmpbuf, "<span font_desc='80' face='%s' color='%s'>%s</span>",
                      config.kana_font_face,
                      config.kana_color,
                      kana_signs[3*number+HIRAGANA]);
@@ -243,7 +250,7 @@ gui_display_kana (gint number, gint mode) {
             if (number >= MIXED_SEPARATOR) {
                 number -= MIXED_SEPARATOR;
             }
-            sprintf (tmpbuf, "<span font_desc='80' face='%s' color='%s'>%s</span>", 
+            sprintf (tmpbuf, "<span font_desc='80' face='%s' color='%s'>%s</span>",
                      config.kana_font_face,
                      config.kana_color,
                      kana_signs[3*number+KATAKANA]);
@@ -254,20 +261,14 @@ gui_display_kana (gint number, gint mode) {
                 number -= MIXED_SEPARATOR;
             }
 
-            sprintf(letbuf, "%s", kana_signs[3*number]);
-
-            if (number == 57) {             /* di */
-                sprintf(letbuf, "di, ji");
-            } else if (number == 58) {      /* du */
-                sprintf(letbuf, "du, zu");
-            }
+            sprintf(letbuf, "%s", gettext(kana_signs[3*number]));
 
             if (old_kana_type == HIRAGANA) {
                 sprintf (tmpbuf, "<span font_desc='60' face='%s' color='%s'>%s</span>"
                                  "<span font_desc='40' face='%s' color='%s'>(%s)</span>",
                                  config.kana_font_face,
                                  config.kana_color,
-                                 kana_signs[3*number+HIRAGANA], 
+                                 kana_signs[3*number+HIRAGANA],
                                  config.kana_font_face,
                                  config.romaji_color,
                                  letbuf);
@@ -276,26 +277,26 @@ gui_display_kana (gint number, gint mode) {
                                  "<span font_desc='40' face='%s' color='%s'>(%s)</span>",
                                  config.kana_font_face,
                                  config.kana_color,
-                                 kana_signs[3*number+KATAKANA], 
+                                 kana_signs[3*number+KATAKANA],
                                  config.kana_font_face,
                                  config.romaji_color,
                                  letbuf);
             } else {
-                sprintf (tmpbuf, "<span font_desc='80' face='%s' color='%s'>%s</span>", 
+                sprintf (tmpbuf, "<span font_desc='80' face='%s' color='%s'>%s</span>",
                          config.kana_font_face,
                          config.romaji_color,
-                         kana_signs[3*number]);
+                         gettext(kana_signs[3*number]));
             }
             break;
         case MIXED:
             if (number >= MIXED_SEPARATOR) {
-                sprintf (tmpbuf, "<span font_desc='80' face='%s' color='%s'>%s</span>", 
+                sprintf (tmpbuf, "<span font_desc='80' face='%s' color='%s'>%s</span>",
                          config.kana_font_face,
                          config.kana_color,
                          kana_signs[3*(number-MIXED_SEPARATOR)+KATAKANA]);
                 old_kana_type = KATAKANA;
             } else {
-                sprintf (tmpbuf, "<span font_desc='80' face='%s' color='%s'>%s</span>", 
+                sprintf (tmpbuf, "<span font_desc='80' face='%s' color='%s'>%s</span>",
                          config.kana_font_face,
                          config.kana_color,
                          kana_signs[3*number+HIRAGANA]);
@@ -312,7 +313,19 @@ gui_display_kana (gint number, gint mode) {
 
 /*--------------------------------------------------------------------*/
 
-gint 
+void
+gui_close_window_cb (GtkWidget *widget, gpointer data) {
+
+    gtk_window_get_size (GTK_WINDOW(main_window),
+                        &config.window_size_x, &config.window_size_y);
+    gdk_window_get_root_origin (main_window->window,
+                                &config.window_x, &config.window_y);
+    gtk_main_quit ();
+}
+
+/*--------------------------------------------------------------------*/
+
+gint
 gui_rm_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer data) {
 
     switch (event->keyval) {
@@ -332,7 +345,7 @@ gui_rm_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer data) {
     return FALSE;
 }
 
-gint 
+gint
 gui_mw_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer data) {
 
     if (test_state == FALSE) {
@@ -350,6 +363,11 @@ gui_mw_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer data) {
             case GDK_s:
                 stats_create_window ();
                 return TRUE;
+            case GDK_q:
+                if (event->state & GDK_CONTROL_MASK) {  /* CTRL + q */
+                    gui_close_window_cb (NULL, NULL);
+                }
+                return TRUE;
         }
     }
 
@@ -357,43 +375,29 @@ gui_mw_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer data) {
     return FALSE;
 }
 
-
-/*--------------------------------------------------------------------*/
-
-void 
-gui_close_window_cb (GtkWidget *widget, gpointer data) {
-
-    gtk_window_get_size (GTK_WINDOW(main_window),
-                        &config.window_size_x, &config.window_size_y);
-    gdk_window_get_root_origin (main_window->window,
-                                &config.window_x, &config.window_y);
-    gtk_main_quit ();
-}
-
 /*--------------------------------------------------------------------*/
 
 void
 gui_combobox_kana_handler_cb (GtkComboBox *widget, gpointer user_data) {
 
     config.kana_mode = HIRAGANA + gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
-} 
+}
 
 void
 gui_combobox_kana_set_handler_cb (GtkComboBox *widget, gpointer user_data) {
 
     config.kana_set = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
-} 
-
+}
 
 /*--------------------------------------------------------------------*/
 
-void 
+void
 gui_create_window (void) {
 
     GtkWidget       *frame1;
     GtkWidget       *vbox0;
     GtkWidget       *vbox1;
-    GtkWidget       *hseparator;
+    GtkWidget       *hseparator_down;
     GtkWidget       *frame2;
     GtkWidget       *frame;
     GtkWidget       *hbox1;
@@ -447,21 +451,22 @@ gui_create_window (void) {
     logo = gdk_pixbuf_new_from_inline(-1, kanatest_logo, FALSE, NULL);
     gtk_widget_show(logo_area);
     gtk_box_pack_start (GTK_BOX (vbox1), logo_area, TRUE, TRUE, 0);
+    gtk_image_set_from_pixbuf (GTK_IMAGE (logo_area), logo);
 
     char_label = gtk_label_new (NULL);
     gtk_box_pack_start (GTK_BOX (vbox1), char_label, TRUE, TRUE, 0);
 
-    hseparator = gtk_hseparator_new ();
-    gtk_widget_show (hseparator);
-    gtk_box_pack_start (GTK_BOX (vbox1), hseparator, FALSE, TRUE, 0);
+    hseparator_up = gtk_hseparator_new ();
+    gtk_widget_show (hseparator_up);
+    gtk_box_pack_start (GTK_BOX (vbox1), hseparator_up, FALSE, TRUE, 0);
 
     progressbar = GTK_PROGRESS_BAR(gtk_progress_bar_new());
     gtk_widget_show (GTK_WIDGET(progressbar));
     gtk_box_pack_start (GTK_BOX (vbox1), GTK_WIDGET(progressbar), FALSE, FALSE, 0);
 
-    hseparator = gtk_hseparator_new ();
-    gtk_widget_show (hseparator);
-    gtk_box_pack_start (GTK_BOX (vbox1), hseparator, FALSE, TRUE, 0);
+    hseparator_down = gtk_hseparator_new ();
+    gtk_widget_show (hseparator_down);
+    gtk_box_pack_start (GTK_BOX (vbox1), hseparator_down, FALSE, TRUE, 0);
 
     hbox1 = gtk_hbox_new (FALSE, 0);
     gtk_widget_show (hbox1);
@@ -480,7 +485,7 @@ gui_create_window (void) {
     label = gtk_label_new (NULL);
     gtk_widget_show (label);
     gtk_frame_set_label_widget (GTK_FRAME(frame_ro), label);
-    sprintf(buffer, "<b>%s </b>", _("Romaji:"));
+    sprintf(buffer, "<b>%s: </b>", _("Romaji"));
     gtk_label_set_markup (GTK_LABEL (label), buffer);
 
     romaji_entry = gtk_entry_new();
@@ -545,7 +550,6 @@ gui_create_window (void) {
     gtk_container_set_border_width (GTK_CONTAINER (about_button), 2);
     gtk_tooltips_set_tip (tooltips, about_button, _("About"), NULL);
 
-
     hbox1 = gtk_hbox_new (FALSE, 0);
     gtk_widget_show (hbox1);
     gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 8);
@@ -574,7 +578,7 @@ gui_create_window (void) {
     label_ka = gtk_label_new (NULL);
     gtk_widget_show (label_ka);
     gtk_frame_set_label_widget (GTK_FRAME(frame), label_ka);
-    sprintf(buffer, "<b>%s </b>", _("Test mode:"));
+    sprintf(buffer, "<b>%s: </b>", _("Test mode"));
     gtk_label_set_markup (GTK_LABEL (label_ka), buffer);
 
     frame = gtk_frame_new (NULL);
@@ -601,7 +605,7 @@ gui_create_window (void) {
     label_le = gtk_label_new (NULL);
     gtk_widget_show (label_le);
     gtk_frame_set_label_widget (GTK_FRAME(frame), label_le);
-    sprintf(buffer, "<b>%s </b>", _("Kana set:"));
+    sprintf(buffer, "<b>%s: </b>", _("Kana set"));
     gtk_label_set_markup (GTK_LABEL (label_le), buffer);
 
     hbuttonbox = gtk_hbutton_box_new ();
