@@ -2,7 +2,7 @@
 /*
  * Kanatest
  *
- * Copyright (C) 2001-2004, 2006 Tomasz Mka <pasp@users.sourceforge.net>
+ * Copyright (C) 2001-2004, 2006 Tomasz Maka <pasp@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,12 @@
 #include <gtk/gtk.h>
 #include <config.h>
 #include <locale.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <unistd.h>
+#include <stdlib.h>
+
+#include <config.h>
 
 #include "gui.h"
 #include "prefs.h"
@@ -33,17 +39,20 @@
 
 int main (int argc, char **argv)
 {
- 
+struct timeval timer;
+
     prefs_read_config ();
     stats_read_list ();
 
     gtk_init (&argc, &argv);
 
-#if ENABLE_NLS
     gtk_set_locale();
     bindtextdomain(PACKAGE, LOCALEDIR);
     textdomain(PACKAGE);
-#endif
+
+    /* set seed */
+    gettimeofday(&timer, NULL);
+    srand48(timer.tv_usec + getpid());
 
     gui_create_window ();
     gtk_main ();
