@@ -2,7 +2,7 @@
 /*
  * Kanatest
  *
- * Copyright (C) 2001-2004, 2006 Tomasz Mąka <pasp@users.sourceforge.net>
+ * Copyright (C) 2001-2004, 2006 Tomasz Maka <pasp@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,7 +38,9 @@
 
 
 GtkWidget *options_window;
-GtkWidget *rwa_option_button;
+GtkWidget *rna_radio_button;
+GtkWidget *rwa_radio_button;
+GtkWidget *raa_radio_button;
 GtkWidget *enable_tooltips_button;
 GtkWidget *example_label;
 
@@ -54,6 +56,11 @@ GtkWidget *number_of_kanas_label;
 GtkWidget *check_buttons[NUMBER_OF_SIGNS];
 gboolean disable_toggles;
 
+GtkWidget *row_button_s[CHART_ROWS], *row_button_c[CHART_ROWS];
+gchar row_pos[CHART_ROWS][CHART_COLUMNS];
+
+/*--------------------------------------------------------------------*/
+
 void
 options_font_select_handler_cb (GtkWidget *widget) {
 
@@ -68,7 +75,7 @@ options_font_select_handler_cb (GtkWidget *widget) {
     gtk_window_set_transient_for(GTK_WINDOW(font_selector), GTK_WINDOW(options_window));
     sprintf(tmpbuf, "%s 16", config.kana_font_face);
     gtk_font_selection_dialog_set_font_name (GTK_FONT_SELECTION_DIALOG(font_selector), tmpbuf);
-    gtk_font_selection_dialog_set_preview_text (GTK_FONT_SELECTION_DIALOG (font_selector), 
+    gtk_font_selection_dialog_set_preview_text (GTK_FONT_SELECTION_DIALOG (font_selector),
                                                 EXAMPLE_KANA_TEXT);
     gtk_widget_show (font_selector);
 
@@ -86,25 +93,30 @@ options_font_select_handler_cb (GtkWidget *widget) {
     gtk_widget_destroy (font_selector);
 }
 
+/*--------------------------------------------------------------------*/
 
-void 
-options_repeat_wrong_changed_cb (GtkToggleButton *button, gpointer user_data) {
+void
+options_repeat_mode_changed_cb (GtkToggleButton *button, gpointer user_data) {
 
-    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(button)) == TRUE)
-        config.repeat_wrong = TRUE;
-    else
-        config.repeat_wrong = FALSE;
+     if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(rna_radio_button)) == TRUE) {
+        config.repeat_mode = REPEAT_NONE;
+     } else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(rwa_radio_button)) == TRUE) {
+        config.repeat_mode = REPEAT_WRONG;
+     } else {
+        config.repeat_mode = REPEAT_ALL;
+     }
 }
 
-void 
+/*--------------------------------------------------------------------*/
+
+void
 options_tooltips_changed_cb (GtkToggleButton *button, gpointer user_data) {
 
-    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(button)) == TRUE)
-        config.enable_tooltips = TRUE;
-    else
-        config.enable_tooltips = FALSE;
+    config.enable_tooltips = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(button));
 }
 
+/*--------------------------------------------------------------------*/
+
 gint
 update_selected_kanas(void) {
 
@@ -123,11 +135,13 @@ update_selected_kanas(void) {
     return n;
 }
 
-void 
+/*--------------------------------------------------------------------*/
+
+void
 kana_selected_cb (GtkToggleButton *button, gpointer user_data) {
 
     gint pos;
-    
+
     if (disable_toggles == FALSE) {
 
         pos = (int) user_data;
@@ -142,11 +156,16 @@ kana_selected_cb (GtkToggleButton *button, gpointer user_data) {
     }
 }
 
-gint delete_event_cb( GtkWidget *widget, GdkEvent *event, gpointer data) {
+/*--------------------------------------------------------------------*/
+
+gint
+delete_event_cb( GtkWidget *widget, GdkEvent *event, gpointer data) {
     return TRUE;
 }
 
-void 
+/*--------------------------------------------------------------------*/
+
+void
 options_window_close_cb (void) {
 
     GtkWidget *info_dialog;
@@ -157,9 +176,9 @@ options_window_close_cb (void) {
 
         gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 1);
 
-        info_dialog = gtk_message_dialog_new (GTK_WINDOW(options_window), 
+        info_dialog = gtk_message_dialog_new (GTK_WINDOW(options_window),
                                               GTK_DIALOG_MODAL,
-                                              GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, 
+                                              GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
                                               _("Please select at least two kanas to test."));
 
         gtk_window_set_transient_for(GTK_WINDOW(info_dialog), GTK_WINDOW(options_window));
@@ -171,12 +190,12 @@ options_window_close_cb (void) {
     } else {
 
         gtk_color_button_get_color(GTK_COLOR_BUTTON(kana_colorbutton), &color);
-        sprintf(color_str, "#%02X%02X%02X", 
+        sprintf(color_str, "#%02X%02X%02X",
                 color.red * 256 / 65536, color.green * 256 / 65536, color.blue * 256 / 65536);
         strncpy(config.kana_color, color_str, MAX_COLORNAME_LEN-1);
 
         gtk_color_button_get_color(GTK_COLOR_BUTTON(romaji_colorbutton), &color);
-        sprintf(color_str, "#%02X%02X%02X", 
+        sprintf(color_str, "#%02X%02X%02X",
                 color.red * 256 / 65536, color.green * 256 / 65536, color.blue * 256 / 65536);
         strncpy(config.romaji_color, color_str, MAX_COLORNAME_LEN-1);
 
@@ -192,14 +211,17 @@ options_window_close_cb (void) {
     }
 }
 
+/*--------------------------------------------------------------------*/
+
 void
 options_combobox_ca_timeout_handler_cb (GtkComboBox *widget, gpointer user_data) {
 
     config.ca_timeout = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
-} 
+}
 
+/*--------------------------------------------------------------------*/
 
-gint 
+gint
 options_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer data) {
 
     if (event->keyval == GDK_Escape) {
@@ -210,8 +232,39 @@ options_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer data) {
     return FALSE;
 }
 
+/*--------------------------------------------------------------------*/
+
 void
-select_action_cb (GtkComboBox *widget, gpointer user_data) {
+options_combobox_dtf_handler_cb (GtkComboBox *widget, gpointer user_data) {
+
+gint i, j, n, pos;
+gchar tmp[BUFFER_SIZE];
+
+    n = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
+    pos = 0;
+
+    for(j=0; j < CHART_ROWS; j++) {
+        for(i=0; i < CHART_COLUMNS; i++) {
+            if(kana_signs_mask[j][i]) {
+                if (n != 0) {
+                    sprintf(tmp, "<span font_desc='12' face='%s'>%s</span>",
+                            config.kana_font_face, gettext(kana_signs[pos*3+n]));
+                } else {
+                    sprintf(tmp, "<span font_desc='10' face='%s'>%s</span>",
+                            config.kana_font_face, gettext(kana_signs[pos*3]));
+                }
+                gtk_label_set_markup (GTK_LABEL(GTK_BIN(GTK_BUTTON(check_buttons[pos]))->child), tmp);
+                pos++;
+            }
+        }
+    }
+
+}
+
+/*--------------------------------------------------------------------*/
+
+void
+select_action_cb (GtkWidget *widget, gpointer user_data) {
 
     gint i, j, pos;
     gboolean state;
@@ -235,7 +288,7 @@ select_action_cb (GtkComboBox *widget, gpointer user_data) {
 
                     case SELECT_INVERT:
                         state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(check_buttons[pos]));
-                        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check_buttons[pos]), (state == TRUE) ? FALSE : TRUE);
+                        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check_buttons[pos]), !state);
                         break;
 
                     default:
@@ -246,9 +299,34 @@ select_action_cb (GtkComboBox *widget, gpointer user_data) {
             }
         }
     }
-} 
+}
 
-void 
+/*--------------------------------------------------------------------*/
+
+void
+select_deselect_row (gint row, gboolean state) {
+gint i;
+
+    for(i=0; i < CHART_COLUMNS; i++) {
+        if (row_pos[row][i] != -1) {
+            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check_buttons[(gint)row_pos[row][i]]), state);
+        }
+    }
+}
+
+void
+row_button_s_cb (GtkWidget *widget, gpointer data) {
+    select_deselect_row ((gint)data, TRUE);
+}
+
+void
+row_button_c_cb (GtkWidget *widget, gpointer data) {
+    select_deselect_row ((gint)data, FALSE);
+}
+
+/*--------------------------------------------------------------------*/
+
+void
 options_create_window (void) {
 
     GtkWidget       *vbox1;
@@ -264,6 +342,7 @@ options_create_window (void) {
     GtkWidget       *frame;
     GtkWidget       *label;
     GtkWidget       *combobox_ca_timeout;
+    GtkWidget       *combobox_dtf;
     GtkWidget       *font_button;
     GtkWidget       *table;
     GdkColor        color;
@@ -276,14 +355,13 @@ options_create_window (void) {
     gint            i, j, pos;
     gchar           buffer[BUFFER_SIZE], tmp[BUFFER_SIZE];
 
-
     options_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
     gtk_window_set_transient_for(GTK_WINDOW(options_window),GTK_WINDOW(main_window));
     gtk_window_set_resizable (GTK_WINDOW (options_window), TRUE);
     gtk_window_set_modal(GTK_WINDOW(options_window), TRUE);
     gtk_container_set_border_width (GTK_CONTAINER (options_window), 12);
     gtk_window_set_title (GTK_WINDOW (options_window), _("Options"));
-    gtk_window_set_default_size (GTK_WINDOW(options_window), 480, 400);
+    gtk_window_set_default_size (GTK_WINDOW(options_window), 560, 460);
 
     g_signal_connect (G_OBJECT (options_window), "delete_event",
                       G_CALLBACK(delete_event_cb), NULL);
@@ -318,7 +396,7 @@ options_create_window (void) {
     gtk_box_pack_start (GTK_BOX (vbox_n), frame, TRUE, FALSE, 0);
     gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
 
-    sprintf(buffer, "<b>%s</b>", _("Drill options:"));
+    sprintf(buffer, "<b>%s:</b>", _("Drill options"));
     label = gtk_label_new (buffer);
     gtk_widget_show (label);
     gtk_frame_set_label_widget (GTK_FRAME (frame), label);
@@ -333,20 +411,40 @@ options_create_window (void) {
     gtk_widget_show (vbox2);
     gtk_container_add (GTK_CONTAINER (alignment), vbox2);
 
-    rwa_option_button = gtk_check_button_new_with_label(_("Repeat wrongly-answered questions"));
-    gtk_container_set_border_width (GTK_CONTAINER (rwa_option_button), 8);
-    g_signal_connect (G_OBJECT (rwa_option_button), "toggled",
-                      G_CALLBACK (options_repeat_wrong_changed_cb), NULL);
-    gtk_box_pack_start (GTK_BOX (vbox2), rwa_option_button, TRUE, TRUE, 0);
-    gtk_widget_show (rwa_option_button);
-    gtk_tooltips_set_tip (tooltips, rwa_option_button, _("When this option is enabled the kanatest will repeat all wrongly answered questions at the end of test"), NULL);
+    rna_radio_button = gtk_radio_button_new_with_label(NULL,
+                                                       _("Display each kana once"));
+    gtk_box_pack_start (GTK_BOX (vbox2),rna_radio_button, TRUE, TRUE, 0);
+    gtk_container_set_border_width (GTK_CONTAINER (rna_radio_button), 8);
+    gtk_widget_show (rna_radio_button);
+    gtk_tooltips_set_tip (tooltips, rna_radio_button, _("When this option is enabled the kanatest will display every question once"), NULL);
+    g_signal_connect (G_OBJECT (rna_radio_button), "toggled",
+                      G_CALLBACK (options_repeat_mode_changed_cb), NULL);
+
+    rwa_radio_button = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(rna_radio_button),
+                                                                   _("Repeat wrongly-answered questions"));
+    gtk_box_pack_start (GTK_BOX (vbox2),rwa_radio_button, TRUE, TRUE, 0);
+    gtk_container_set_border_width (GTK_CONTAINER (rwa_radio_button), 8);
+    gtk_widget_show (rwa_radio_button);
+    gtk_tooltips_set_tip (tooltips, rwa_radio_button, _("When this option is enabled the kanatest will repeat all wrongly answered questions at the end of test"), NULL);
+    g_signal_connect (G_OBJECT (rwa_radio_button), "toggled",
+                      G_CALLBACK (options_repeat_mode_changed_cb), NULL);
+
+    raa_radio_button = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(rna_radio_button),
+                                                                   _("Repeat all forever"));
+    gtk_box_pack_start (GTK_BOX (vbox2),raa_radio_button, TRUE, TRUE, 0);
+    gtk_container_set_border_width (GTK_CONTAINER (raa_radio_button), 8);
+    gtk_widget_show (raa_radio_button);
+    gtk_tooltips_set_tip (tooltips, raa_radio_button, _("When this option is enabled the kanatest will repeat all questions forever"), NULL);
+    g_signal_connect (G_OBJECT (raa_radio_button), "toggled",
+                      G_CALLBACK (options_repeat_mode_changed_cb), NULL);
 
     table = gtk_table_new (2, 1, FALSE);
     gtk_widget_show (table);
     gtk_container_set_border_width (GTK_CONTAINER (table), 12);
     gtk_box_pack_start (GTK_BOX (vbox2), table, FALSE, FALSE, 0);
 
-    label = gtk_label_new (_("Correct answer display timeout:"));
+    sprintf(buffer, "%s:", _("Correct answer display timeout"));
+    label = gtk_label_new (buffer);
     gtk_widget_show (label);
     gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
                     (GtkAttachOptions) (GTK_FILL),
@@ -377,7 +475,7 @@ options_create_window (void) {
     gtk_box_pack_start (GTK_BOX (vbox_n), frame, TRUE, FALSE, 0);
     gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
 
-    sprintf(buffer, "<b>%s</b>", _("Appearance:"));
+    sprintf(buffer, "<b>%s:</b>", _("Appearance"));
     label = gtk_label_new (buffer);
     gtk_widget_show (label);
     gtk_frame_set_label_widget (GTK_FRAME (frame), label);
@@ -397,7 +495,8 @@ options_create_window (void) {
     gtk_container_set_border_width (GTK_CONTAINER (table), 12);
     gtk_box_pack_start (GTK_BOX (vbox2), table, TRUE, TRUE, 0);
 
-    label = gtk_label_new (_("Kana color:"));
+    sprintf(buffer, "%s:", _("Kana color"));
+    label = gtk_label_new (buffer);
     gtk_widget_show (label);
     gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
                     (GtkAttachOptions) (GTK_FILL),
@@ -414,7 +513,8 @@ options_create_window (void) {
     gdk_color_parse(config.kana_color, &color);
     gtk_color_button_set_color (GTK_COLOR_BUTTON (kana_colorbutton), &color);
 
-    label = gtk_label_new (_("Romaji color:"));
+    sprintf(buffer, "%s:", _("Romaji color"));
+    label = gtk_label_new (buffer);
     gtk_widget_show (label);
     gtk_table_attach (GTK_TABLE (table), label, 2, 3, 0, 1,
                     (GtkAttachOptions) (GTK_FILL),
@@ -447,7 +547,7 @@ options_create_window (void) {
     gtk_box_pack_start (GTK_BOX (vbox_n), frame, TRUE, FALSE, 0);
     gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
 
-    sprintf(buffer, "<b>%s</b>", _("Misc options:"));
+    sprintf(buffer, "<b>%s:</b>", _("Misc options"));
     label = gtk_label_new (buffer);
     gtk_widget_show (label);
     gtk_frame_set_label_widget (GTK_FRAME (frame), label);
@@ -492,7 +592,7 @@ options_create_window (void) {
     gtk_widget_show (vbox2);
     gtk_container_add (GTK_CONTAINER (viewport), vbox2);
 
-    table = gtk_table_new (CHART_ROWS, CHART_COLUMNS, FALSE);
+    table = gtk_table_new (CHART_ROWS, CHART_COLUMNS+2, FALSE);     /* 2 - S and C buttons */
     gtk_widget_show (table);
     gtk_box_pack_start (GTK_BOX (vbox2), table, FALSE, TRUE, 0);
 
@@ -502,6 +602,7 @@ options_create_window (void) {
     for(j=0; j < CHART_ROWS; j++) {
         for(i=0; i < CHART_COLUMNS; i++) {
 
+            row_pos[j][i] = -1;
             if(kana_signs_mask[j][i]) {
 
                 frames[pos] = gtk_frame_new (NULL);
@@ -512,17 +613,7 @@ options_create_window (void) {
                 gtk_container_set_border_width (GTK_CONTAINER (frames[pos]), 2);
                 gtk_frame_set_shadow_type (GTK_FRAME (frames[pos]), GTK_SHADOW_ETCHED_OUT);
 
-                if (!strcmp(kana_signs[pos*3+ROMAJI], "di")) {
-
-                    sprintf(tmp, "%s, ji", kana_signs[pos*3+ROMAJI]);
-
-                } else if (!strcmp(kana_signs[pos*3+ROMAJI], "du")) {
-
-                    sprintf(tmp, "%s, zu", kana_signs[pos*3+ROMAJI]);
-
-                } else {
-                    sprintf(tmp, "%s", kana_signs[pos*3+ROMAJI]);
-                }
+                sprintf(tmp, "%s", gettext(kana_signs[pos*3+ROMAJI]));
 
                 check_buttons[pos] = gtk_check_button_new_with_mnemonic (tmp);
 
@@ -534,11 +625,40 @@ options_create_window (void) {
 
                 if(config.user_defined_lesson[pos] == '+') {
                     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check_buttons[pos]), TRUE);
-                } 
+                }
 
+                row_pos[j][i] = pos;
                 pos++;
             }
+
         }
+
+        i++;
+        row_button_s[j] = gui_stock_label_button(NULL, GTK_STOCK_APPLY);
+        gtk_button_set_relief (GTK_BUTTON (row_button_s[j]), GTK_RELIEF_NONE);
+        GTK_WIDGET_UNSET_FLAGS(row_button_s[j], GTK_CAN_FOCUS);
+        gtk_tooltips_set_tip (tooltips, row_button_s[j], _("Select all kanas in row"), NULL);
+        gtk_widget_show (row_button_s[j]);
+        gtk_table_attach (GTK_TABLE (table), row_button_s[j], i, i+1, j, j+1,
+                          (GtkAttachOptions) (GTK_FILL),
+                          (GtkAttachOptions) (GTK_FILL), 0, 0);
+
+        g_signal_connect (G_OBJECT (row_button_s[j]), "clicked",
+                            G_CALLBACK (row_button_s_cb), (gpointer) j);
+
+        i++;
+        row_button_c[j] = gui_stock_label_button(NULL, GTK_STOCK_CLEAR);
+        gtk_button_set_relief (GTK_BUTTON (row_button_c[j]), GTK_RELIEF_NONE);
+        GTK_WIDGET_UNSET_FLAGS(row_button_c[j], GTK_CAN_FOCUS);
+        gtk_tooltips_set_tip (tooltips, row_button_c[j], _("Deselect all kanas in row"), NULL);
+        gtk_widget_show (row_button_c[j]);
+        gtk_table_attach (GTK_TABLE (table), row_button_c[j], i, i+1, j, j+1,
+                          (GtkAttachOptions) (GTK_FILL),
+                          (GtkAttachOptions) (GTK_FILL), 0, 0);
+
+        g_signal_connect (G_OBJECT (row_button_c[j]), "clicked",
+                            G_CALLBACK (row_button_c_cb), (gpointer) j);
+
     }
 
     disable_toggles = FALSE;
@@ -547,7 +667,8 @@ options_create_window (void) {
     gtk_widget_show (hbox);
     gtk_box_pack_start (GTK_BOX (vbox_l), hbox, FALSE, TRUE, 4);
 
-    label = gtk_label_new (_("<b>Selected kanas:</b>"));
+    sprintf(buffer, "<b>%s:</b>", _("Selected kanas"));
+    label = gtk_label_new (buffer);
     gtk_widget_show (label);
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
     gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
@@ -558,28 +679,56 @@ options_create_window (void) {
     gtk_box_pack_start (GTK_BOX (hbox), number_of_kanas_label, FALSE, FALSE, 0);
     gtk_misc_set_padding (GTK_MISC (number_of_kanas_label), 4, 0);
 
+    hbox = gtk_hbox_new (FALSE, 0);
+    gtk_widget_show (hbox);
+    gtk_box_pack_start (GTK_BOX (vbox_l), hbox, FALSE, TRUE, 4);
+
+    sprintf(buffer, "<b>%s:</b>", _("Display as"));
+    label = gtk_label_new (buffer);
+    gtk_widget_show (label);
+    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+    gtk_misc_set_padding (GTK_MISC (label), 4, 0);
+
+    combobox_dtf = gtk_combo_box_new_text ();
+    GTK_WIDGET_UNSET_FLAGS(combobox_dtf, GTK_CAN_FOCUS);
+    gtk_widget_show (combobox_dtf);
+    gtk_box_pack_start (GTK_BOX (hbox), combobox_dtf, FALSE, FALSE, 0);
+    g_signal_connect (G_OBJECT (combobox_dtf), "changed",
+                      G_CALLBACK (options_combobox_dtf_handler_cb), NULL);
+    gtk_combo_box_append_text (GTK_COMBO_BOX (combobox_dtf), _("Romaji"));
+    gtk_combo_box_append_text (GTK_COMBO_BOX (combobox_dtf), _("Hiragana"));
+    gtk_combo_box_append_text (GTK_COMBO_BOX (combobox_dtf), _("Katakana"));
+    gtk_combo_box_set_active (GTK_COMBO_BOX (combobox_dtf), 0);
+
     hbuttonbox_s = gtk_hbutton_box_new ();
     gtk_widget_show (hbuttonbox_s);
-    gtk_box_pack_start (GTK_BOX (vbox_l), hbuttonbox_s, FALSE, TRUE, 0);
-    gtk_box_set_spacing (GTK_BOX (hbuttonbox_s), 5);
-    gtk_container_set_border_width (GTK_CONTAINER (hbuttonbox_s), 4);
+    gtk_box_pack_end (GTK_BOX (hbox), hbuttonbox_s, FALSE, TRUE, 0);
+    gtk_box_set_spacing (GTK_BOX (hbuttonbox_s), 4);
     gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox_s), GTK_BUTTONBOX_END);
 
-    select_all_button = gtk_button_new_with_label (_("Select all"));
+    sprintf(buffer, "<b>%s:</b>", _("Select"));
+    label = gtk_label_new (buffer);
+    gtk_widget_show (label);
+    gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+    gtk_misc_set_padding (GTK_MISC (label), 4, 0);
+
+    select_all_button = gtk_button_new_with_label (_("All"));
     GTK_WIDGET_UNSET_FLAGS(select_all_button, GTK_CAN_FOCUS);
     gtk_widget_show (select_all_button);
     gtk_container_add (GTK_CONTAINER (hbuttonbox_s), select_all_button);
     g_signal_connect (G_OBJECT (select_all_button), "clicked",
                         G_CALLBACK (select_action_cb), (gpointer) SELECT_ALL);
 
-    select_none_button = gtk_button_new_with_label (_("Select none"));
+    select_none_button = gtk_button_new_with_label (_("None"));
     GTK_WIDGET_UNSET_FLAGS(select_none_button, GTK_CAN_FOCUS);
     gtk_widget_show (select_none_button);
     gtk_container_add (GTK_CONTAINER (hbuttonbox_s), select_none_button);
     g_signal_connect (G_OBJECT (select_none_button), "clicked",
                         G_CALLBACK (select_action_cb), (gpointer) SELECT_NONE);
 
-    invert_selection_button = gtk_button_new_with_label (_("Invert selection"));
+    invert_selection_button = gtk_button_new_with_label (_("Invert"));
     GTK_WIDGET_UNSET_FLAGS(invert_selection_button, GTK_CAN_FOCUS);
     gtk_widget_show (select_none_button);
     gtk_widget_show (invert_selection_button);
@@ -587,7 +736,6 @@ options_create_window (void) {
     g_signal_connect (G_OBJECT (invert_selection_button), "clicked",
                         G_CALLBACK (select_action_cb), (gpointer) SELECT_INVERT);
 
-
     hseparator = gtk_hseparator_new ();
     gtk_widget_show (hseparator);
     gtk_box_pack_start (GTK_BOX (vbox1), hseparator, FALSE, FALSE, 8);
@@ -604,8 +752,15 @@ options_create_window (void) {
     gtk_container_add (GTK_CONTAINER (hbuttonbox), close_button);
     GTK_WIDGET_SET_FLAGS (close_button, GTK_CAN_DEFAULT);
 
-    if (config.repeat_wrong == TRUE) {
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rwa_option_button), TRUE);
+    switch(config.repeat_mode) {
+        case REPEAT_ALL:
+            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(raa_radio_button), TRUE);
+            break;
+        case REPEAT_WRONG:
+            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rwa_radio_button), TRUE);
+            break;
+        default:
+            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(rna_radio_button), TRUE);
     }
 
     if (config.enable_tooltips == TRUE) {
@@ -619,3 +774,5 @@ options_create_window (void) {
     gtk_widget_grab_default (close_button);
 }
 
+/*--------------------------------------------------------------------*/
+
