@@ -2,7 +2,7 @@
 /*
  * Kanatest
  *
- * Copyright (C) 2001-2004, 2006 Tomasz Mka <pasp@users.sourceforge.net>
+ * Copyright (C) 2001-2004, 2006 Tomasz Maka <pasp@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include <unistd.h>
 #include <sys/stat.h>
 #include <limits.h>
+#include <glib/gstdio.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 
@@ -35,35 +36,40 @@
 
 struct jpprefs config;
 
-gchar* 
-prefs_get_config_filename (void)
-{
-static gchar dirname[PATH_MAX], filename[PATH_MAX];
-struct stat cfg;
+/*--------------------------------------------------------------------*/
 
-    strncpy (dirname, getenv ("HOME"), PATH_MAX);
-    strcat (dirname, "/");
+gchar*
+prefs_get_config_filename (gchar *f_name) {
+
+static gchar filename[PATH_MAX];
+gchar dirname[PATH_MAX]; 
+gchar separator[2] = { G_DIR_SEPARATOR, '\0' };
+
+    strncpy (dirname, g_get_home_dir(), PATH_MAX-1);
+    strcat (dirname, separator);
     strcat (dirname, CONFIG_DIRNAME);
 
-    if(stat (dirname, &cfg) < 0)
-        mkdir (dirname, S_IRUSR | S_IWUSR | S_IXUSR);
+    if (g_file_test(dirname, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR) != TRUE) {
+        g_mkdir (dirname, S_IRUSR | S_IWUSR | S_IXUSR);
+    }
 
     strcpy (filename, dirname);
-    strcat (filename, "/");
-    strcat (filename, CONFIG_FILENAME);
+    strcat (filename, separator);
+    strcat (filename, f_name);
 
     return filename;
 }
 
+/*--------------------------------------------------------------------*/
+
+void
+prefs_set_default_values (void) {
 
-void 
-prefs_set_default_values (void)
-{
     config.window_x = 180;
     config.window_y = 120;
     config.window_size_x = 365;
     config.window_size_y = 445;
-    config.repeat_wrong = FALSE;
+    config.repeat_mode = REPEAT_NONE;
     config.kana_mode = HIRAGANA;
     config.kana_set = 0;            /* all kanas */
     config.ca_timeout = 1;          /* 1 sec */
@@ -86,21 +92,22 @@ prefs_set_default_values (void)
     strncpy(config.user_defined_lesson, "+++++----------------------------------------------------------------------------------------------------", NUMBER_OF_SIGNS);
 }
 
+/*--------------------------------------------------------------------*/
+
+void
+prefs_read_config (void) {
 
-void 
-prefs_read_config (void)
-{
 gboolean cfg_file;
 xmlDocPtr doc;
 xmlNodePtr node;
 xmlChar *key;
 
-    cfg_file = g_file_test(prefs_get_config_filename(), G_FILE_TEST_IS_REGULAR);
+    cfg_file = g_file_test(prefs_get_config_filename(CONFIG_FILENAME), G_FILE_TEST_IS_REGULAR);
 
     if (cfg_file == TRUE) {
 
-        doc = xmlParseFile(prefs_get_config_filename());
-        if (doc == NULL) 
+        doc = xmlParseFile(prefs_get_config_filename(CONFIG_FILENAME));
+        if (doc == NULL)
                 return;
 
         node = xmlDocGetRootElement(doc);
@@ -115,7 +122,7 @@ xmlChar *key;
         }
 
         prefs_set_default_values();
-       
+
         node = node->xmlChildrenNode;
 
         while (node != NULL) {
@@ -148,10 +155,10 @@ xmlChar *key;
                     xmlFree(key);
                 }
 
-                if ((!xmlStrcmp(node->name, (const xmlChar *) "repeat_wrong"))) {
+                if ((!xmlStrcmp(node->name, (const xmlChar *) "repeat_mode"))) {
                     key = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
                     if (key != NULL)
-                            sscanf((gchar *) key, "%d", &config.repeat_wrong);
+                            sscanf((gchar *) key, "%d", &config.repeat_mode);
                     xmlFree(key);
                 }
 
@@ -290,7 +297,7 @@ xmlChar *key;
 
                 if ((!xmlStrcmp(node->name, (const xmlChar *)"user_defined_lesson"))) {
                     key = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
-                    if (key != NULL) 
+                    if (key != NULL)
                     {
                             strncpy(config.user_defined_lesson, (gchar *) key, NUMBER_OF_SIGNS);
                             config.user_defined_lesson[NUMBER_OF_SIGNS+1] = '\0';
@@ -298,8 +305,6 @@ xmlChar *key;
                     xmlFree(key);
                 }
 
-
-
             node = node->next;
         }
 
@@ -313,9 +318,11 @@ xmlChar *key;
 
 }
 
+/*--------------------------------------------------------------------*/
+
+void
+prefs_write_config(void) {
 
-void prefs_write_config(void)
-{
 xmlDocPtr doc;
 xmlNodePtr node;
 gchar temp[MAX_BUFFER_SIZE];
@@ -332,8 +339,8 @@ gchar temp[MAX_BUFFER_SIZE];
     xmlNewTextChild(node, NULL, (const xmlChar *) "window_size_x", (xmlChar *) temp);
     snprintf(temp, 32, "%d", config.window_size_y);
     xmlNewTextChild(node, NULL, (const xmlChar *) "window_size_y", (xmlChar *) temp);
-    snprintf(temp, 32, "%d", config.repeat_wrong);
-    xmlNewTextChild(node, NULL, (const xmlChar *) "repeat_wrong", (xmlChar *) temp);
+    snprintf(temp, 32, "%d", config.repeat_mode);
+    xmlNewTextChild(node, NULL, (const xmlChar *) "repeat_mode", (xmlChar *) temp);
     snprintf(temp, 32, "%d", config.kana_mode);
     xmlNewTextChild(node, NULL, (const xmlChar *) "kana_mode", (xmlChar *) temp);
     snprintf(temp, 32, "%d", config.kana_set);
@@ -375,8 +382,9 @@ gchar temp[MAX_BUFFER_SIZE];
     snprintf(temp, NUMBER_OF_SIGNS+1, "%s", config.user_defined_lesson);
     xmlNewTextChild(node, NULL, (const xmlChar *) "user_defined_lesson", (xmlChar *) temp);
 
-    xmlSaveFormatFile(prefs_get_config_filename(), doc, 1);
+    xmlSaveFormatFile(prefs_get_config_filename(CONFIG_FILENAME), doc, 1);
     xmlFreeDoc(doc);
 }
 
+/*--------------------------------------------------------------------*/
 
