@@ -2,7 +2,7 @@
 /*
  * Kanatest
  *
- * Copyright (C) 2001-2004, 2006 Tomasz Mka <pasp@users.sourceforge.net>
+ * Copyright (C) 2001-2004, 2006 Tomasz Maka <pasp@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ struct jpprefs
     gint        window_y;
     gint        window_size_x;
     gint        window_size_y;
-    gint        repeat_wrong;
+    gint        repeat_mode;
     gint        kana_mode;
     gint        kana_set;
     gint        ca_timeout;
@@ -57,16 +57,17 @@ struct jpprefs
     gint        chart_window_y;
     gint        stats_sort_column;
     gint        stats_sort_column_dir;
-   	gchar       kana_font_face[MAX_FONTNAME_LEN];
-   	gchar       kana_color[MAX_COLORNAME_LEN];
-   	gchar       romaji_color[MAX_COLORNAME_LEN];
-   	gchar       user_defined_lesson[NUMBER_OF_SIGNS+1];
+    gchar       kana_font_face[MAX_FONTNAME_LEN];
+    gchar       kana_color[MAX_COLORNAME_LEN];
+    gchar       romaji_color[MAX_COLORNAME_LEN];
+    gchar       user_defined_lesson[NUMBER_OF_SIGNS+1];
 };
 
 extern  struct jpprefs        config;
 
-void    prefs_read_config       (void);
-void    prefs_write_config      (void);
+void    prefs_read_config           (void);
+void    prefs_write_config          (void);
+gchar*  prefs_get_config_filename   (gchar *f_name);
 
 #endif /* _PREFS_H */
 
