@@ -2,7 +2,7 @@
 /*
  * Kanatest
  *
- * Copyright (C) 2001-2004, 2006 Tomasz Mąka <pasp@users.sourceforge.net>
+ * Copyright (C) 2001-2004, 2006 Tomasz Maka <pasp@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,12 +25,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
+#include <limits.h>
+#include <sys/stat.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
-#include <unistd.h>
-#include <sys/stat.h>
-#include <linux/limits.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 
@@ -48,19 +48,21 @@ GSList *stats_list;
 
 extern GtkWidget *main_window;
 
-void 
+/*--------------------------------------------------------------------*/
+
+void
 stats_window_close_cb(void) {
 
     gtk_window_get_size (GTK_WINDOW(stat_window),
                         &config.stat_size_x, &config.stat_size_y);
     gdk_window_get_root_origin (stat_window->window,
                                 &config.stat_window_x, &config.stat_window_y);
-
     gtk_widget_destroy (stat_window);
 }
 
+/*--------------------------------------------------------------------*/
 
-gint 
+gint
 stats_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer data) {
 
     if (event->keyval == GDK_Escape) {
@@ -71,16 +73,17 @@ stats_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer data) {
     return FALSE;
 }
 
+/*--------------------------------------------------------------------*/
 
-void 
+void
 stats_reset_cb (void) {
 
 GtkWidget *info_dialog;
 gint response;
 
-        info_dialog = gtk_message_dialog_new (GTK_WINDOW(stat_window), 
+        info_dialog = gtk_message_dialog_new (GTK_WINDOW(stat_window),
                                               GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
-                                              GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, 
+                                              GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
                                               _("All statistics entries will be destroyed. Continue ?"));
 
         gtk_window_set_title(GTK_WINDOW(info_dialog), _("Question"));
@@ -88,16 +91,18 @@ gint response;
         response = gtk_dialog_run(GTK_DIALOG(info_dialog));
         gtk_widget_destroy(info_dialog);
 
-        if (response == GTK_RESPONSE_YES) {   
+        if (response == GTK_RESPONSE_YES) {
             stats_remove_list ();
             stats_window_close_cb();
         }
 }
 
+/*--------------------------------------------------------------------*/
+
 void
 stats_column_clicked_cb (GtkTreeViewColumn *treeviewcolumn, gpointer user_data) {
 
-    old_column = config.stats_sort_column;  
+    old_column = config.stats_sort_column;
     config.stats_sort_column = (gint) user_data;
 
     if (disable_dir == FALSE) {
@@ -110,7 +115,9 @@ stats_column_clicked_cb (GtkTreeViewColumn *treeviewcolumn, gpointer user_data)
     }
 }
 
-void 
+/*--------------------------------------------------------------------*/
+
+void
 stats_create_window (void) {
 
 GtkWidget       *vbox1;
@@ -133,7 +140,8 @@ struct stats_entry *a;
 GtkTreeViewColumn   *stats_column[NUMBER_OF_COLUMNS];
 
 gchar *column_names[NUMBER_OF_COLUMNS] = {
-    _("Date"), _("Test mode"), _("Test time"), _("Questions"), _("Correct"), _("Wrong"), _("Ratio (%)"), _("Kana set")
+    _("Date"), _("Test mode"), _("Test time"), _("Questions"),
+    _("Correct"), _("Wrong"), _("Ratio (%)"), _("Kana set")
 };
 
 
@@ -154,15 +162,14 @@ gchar *column_names[NUMBER_OF_COLUMNS] = {
         g_signal_connect (G_OBJECT(stat_window), "key_press_event",
                             G_CALLBACK(stats_key_press_cb), NULL);
 
+        gtk_window_move (GTK_WINDOW (stat_window),
+                                    config.stat_window_x, config.stat_window_y);
+
         gtk_window_set_default_size (GTK_WINDOW(stat_window),
                                     config.stat_size_x, config.stat_size_y);
 
         gtk_window_set_resizable (GTK_WINDOW (stat_window), TRUE);
 
-        gtk_window_move (GTK_WINDOW (stat_window),
-                                    config.stat_window_x, config.stat_window_y);
-        gtk_widget_show (stat_window);
-
         vbox1 = gtk_vbox_new (FALSE, 0);
         gtk_widget_show (vbox1);
         gtk_container_add (GTK_CONTAINER (stat_window), vbox1);
@@ -173,14 +180,14 @@ gchar *column_names[NUMBER_OF_COLUMNS] = {
         gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (stats_scrolledwindow), GTK_SHADOW_IN);
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (stats_scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
-        stats_store = gtk_list_store_new (NUMBER_OF_COLUMNS, 
-                                          G_TYPE_STRING, 
-                                          G_TYPE_STRING, 
-                                          G_TYPE_STRING, 
-                                          G_TYPE_INT, 
-                                          G_TYPE_INT, 
-                                          G_TYPE_INT, 
-                                          G_TYPE_INT, 
+        stats_store = gtk_list_store_new (NUMBER_OF_COLUMNS,
+                                          G_TYPE_STRING,
+                                          G_TYPE_STRING,
+                                          G_TYPE_STRING,
+                                          G_TYPE_INT,
+                                          G_TYPE_INT,
+                                          G_TYPE_INT,
+                                          G_TYPE_INT,
                                           G_TYPE_STRING);
 
         stats_model = GTK_TREE_MODEL(stats_store);
@@ -190,7 +197,7 @@ gchar *column_names[NUMBER_OF_COLUMNS] = {
         gtk_tree_view_set_search_column (GTK_TREE_VIEW (stats_treeview), 0);
         gtk_container_add(GTK_CONTAINER (stats_scrolledwindow), stats_treeview);
         gtk_widget_show (stats_treeview);
-        
+
         stats_renderer = gtk_cell_renderer_text_new ();
 
         for (i=0; i < NUMBER_OF_COLUMNS; i++) {
@@ -211,7 +218,6 @@ gchar *column_names[NUMBER_OF_COLUMNS] = {
         gtk_widget_show (hseparator);
         gtk_box_pack_end (GTK_BOX (vbox1), hseparator, FALSE, TRUE, 8);
 
-
         reset_button = gui_stock_label_button(_("Reset stats"), GTK_STOCK_CLEAR);
         gtk_widget_show (reset_button);
         g_signal_connect (G_OBJECT (reset_button), "clicked",
@@ -227,7 +233,6 @@ gchar *column_names[NUMBER_OF_COLUMNS] = {
 
         gtk_widget_grab_default (close_button);
 
-
         /*------------------------------------------------------------------------------*/
 
         if (stats_list != NULL) {
@@ -238,47 +243,55 @@ gchar *column_names[NUMBER_OF_COLUMNS] = {
 
                 gtk_list_store_append (stats_store, &stats_iter);
 
-                sprintf(buffer, "%4d.%02d.%02d, %02d:%02d", a->date_year, a->date_month+1, a->date_day+1, a->date_hour, a->date_minute);
+                sprintf(buffer, "%4d.%02d.%02d, %02d:%02d", a->date_year, a->date_month+1, a->date_day+1,
+                        a->date_hour, a->date_minute);
                 gtk_list_store_set (stats_store, &stats_iter, 0, buffer, -1);
 
-                if (a->test_rwaq == TRUE) {
-                    sprintf(buffer, _("%s (R)"), _(kana_mode_names[a->test_mode]));
-                } else {
-                    sprintf(buffer, _("%s"), _(kana_mode_names[a->test_mode]));
+                switch(a->test_repeat_mode) {
+                    case REPEAT_WRONG:
+                        sprintf(buffer, "%s (%s)", gettext(kana_mode_names[a->test_mode]), _("RW"));
+                        break;
+                    case REPEAT_ALL:
+                        sprintf(buffer, "%s (%s)", gettext(kana_mode_names[a->test_mode]), _("RA"));
+                        break;
+                    default:
+                        sprintf(buffer, "%s", gettext(kana_mode_names[a->test_mode]));
+                        break;
                 }
 
-                gtk_list_store_set (stats_store, &stats_iter, 1, buffer, 
-                                    2, test_sec2str(a->test_time, TRUE), 3, a->test_questions, 
+                gtk_list_store_set (stats_store, &stats_iter, 1, buffer,
+                                    2, test_sec2str(a->test_time, TRUE), 3, a->test_questions,
                                     4, a->test_correct_answers, 5, a->test_questions-a->test_correct_answers,
                                     6, (gint)((gfloat)(a->test_correct_answers)/a->test_questions*100.0),
                                     7, _(kana_signs_names[a->test_kana_set]), -1);
-            }            
+            }
+
+        }
 
-        } 
- 
         disable_dir = TRUE;
 
-        g_signal_emit_by_name(stats_column[config.stats_sort_column], "clicked");   
-
+        g_signal_emit_by_name(stats_column[config.stats_sort_column], "clicked");
 
         if (config.stats_sort_column_dir == -1) {
-            g_signal_emit_by_name(stats_column[config.stats_sort_column], "clicked");   
+            g_signal_emit_by_name(stats_column[config.stats_sort_column], "clicked");
         }
 
         disable_dir = FALSE;
+
+        gtk_widget_show (stat_window);
     }
 }
 
 /*--------------------------------------------------------------------*/
 
-void 
+void
 stats_info_no_stats(void) {
 
 GtkWidget *info_dialog;
 
-        info_dialog = gtk_message_dialog_new (GTK_WINDOW(main_window), 
+        info_dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
                                               GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
-                                              GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE, 
+                                              GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
                                               _("Statistics are not available. The results table is empty."));
 
         gtk_window_set_title(GTK_WINDOW(info_dialog), _("Warning"));
@@ -287,29 +300,6 @@ GtkWidget *info_dialog;
         gtk_widget_destroy(info_dialog);
 }
 
-
-/*--------------------------------------------------------------------*/
-
-gchar* 
-stats_get_filename (void)
-{
-static gchar dirname[PATH_MAX], filename[PATH_MAX];
-struct stat cfg;
-
-    strncpy (dirname, getenv ("HOME"), PATH_MAX);
-    strcat (dirname, "/");
-    strcat (dirname, CONFIG_DIRNAME);
-
-    if(stat (dirname, &cfg) < 0)
-        mkdir (dirname, S_IRUSR | S_IWUSR | S_IXUSR);
-
-    strcpy (filename, dirname);
-    strcat (filename, "/");
-    strcat (filename, STATS_FILENAME);
-
-    return filename;
-}
-
 /*--------------------------------------------------------------------*/
 
 void
@@ -335,7 +325,7 @@ gint i;
 
         while (i >= 0) {
             a = g_slist_nth_data (stats_list, i);
-            stats_list = g_slist_remove (stats_list, a);   
+            stats_list = g_slist_remove (stats_list, a);
             --i;
         }
     }
@@ -355,7 +345,6 @@ stats_get_records (void) {
         return i;
 
     } else {
-
         return 0;
     }
 }
@@ -363,8 +352,8 @@ stats_get_records (void) {
 /*--------------------------------------------------------------------*/
 
 void
-stats_add_entry (guint day, guint month, guint year, guint hour, guint minute, guint time, guint mode, 
-                 guint kana_set, guint questions, guint correct_answers, guint rwaq) {
+stats_add_entry (guint day, guint month, guint year, guint hour, guint minute, guint time, guint mode,
+                 guint kana_set, guint questions, guint correct_answers, guint repeat_mode) {
 
 struct stats_entry *a;
 
@@ -382,18 +371,15 @@ struct stats_entry *a;
         a->test_kana_set = kana_set;
         a->test_questions = questions;
         a->test_correct_answers = correct_answers;
-        a->test_rwaq = rwaq;
+        a->test_repeat_mode = repeat_mode;
 
         stats_list = g_slist_append (stats_list, a);
 
     } else {
-
         fprintf (stderr, "*** ERROR: cannot allocate memory for stats structure\n");
-
     }
 }
 
-
 /*--------------------------------------------------------------------*/
 
 void
@@ -403,11 +389,11 @@ xmlDocPtr doc;
 xmlChar *key;
 xmlNodePtr node, cnode;
 guint day, month, year, hour, minute, time, mode;
-guint kana_set, questions, correct_answers, rwaq;
+guint kana_set, questions, correct_answers, repeat_mode;
 
     stats_list = NULL;
 
-    if((doc = xmlParseFile(stats_get_filename()))) {
+    if((doc = xmlParseFile(prefs_get_config_filename(STATS_FILENAME)))) {
 
         if(!(node = xmlDocGetRootElement(doc))) {
             return;
@@ -425,7 +411,7 @@ guint kana_set, questions, correct_answers, rwaq;
                     cnode = node->xmlChildrenNode;
 
                     day = month = year = hour = minute = time = mode = 0;
-                    kana_set = questions = correct_answers = rwaq = 0;
+                    kana_set = questions = correct_answers = repeat_mode = 0;
 
                     while (cnode != NULL) {
 
@@ -492,15 +478,23 @@ guint kana_set, questions, correct_answers, rwaq;
                         if ((!xmlStrcmp(cnode->name, (const xmlChar *) "test_rwaq"))) {
                             key = xmlNodeListGetString(doc, cnode->xmlChildrenNode, 1);
                             if (key != NULL)
-                                    rwaq = atoi ((gchar *) key);
+                                repeat_mode = (!atoi ((gchar *) key)?REPEAT_NONE:REPEAT_WRONG);
+                            xmlFree(key);
+                        }
+
+                        if ((!xmlStrcmp(cnode->name, (const xmlChar *) "test_repeat_mode"))) {
+                            key = xmlNodeListGetString(doc, cnode->xmlChildrenNode, 1);
+                            if (key != NULL)
+                                repeat_mode = atoi ((gchar *) key);
                             xmlFree(key);
                         }
 
                         cnode = cnode->next;
                     }
 
-                    stats_add_entry (day, month, year, hour, minute, time, mode, kana_set, questions, correct_answers, rwaq);
-           
+                    stats_add_entry (day, month, year, hour, minute, time,
+                                     mode, kana_set, questions, correct_answers, repeat_mode);
+
                 }
 
                 node = node->next;
@@ -555,13 +549,13 @@ gchar temp[BUFFER_SIZE];
         xmlNewChild(stats_node, NULL, (const xmlChar *) "test_questions", (xmlChar *) temp);
         sprintf(temp, "%d", a->test_correct_answers);
         xmlNewChild(stats_node, NULL, (const xmlChar *) "test_correct_answers", (xmlChar *) temp);
-        sprintf(temp, "%d", a->test_rwaq);
-        xmlNewChild(stats_node, NULL, (const xmlChar *) "test_rwaq", (xmlChar *) temp);
+        sprintf(temp, "%d", a->test_repeat_mode);
+        xmlNewChild(stats_node, NULL, (const xmlChar *) "test_repeat_mode", (xmlChar *) temp);
     }
 
     stats_free_list();
 
-    xmlSaveFormatFile(stats_get_filename(), doc, 1);
+    xmlSaveFormatFile(prefs_get_config_filename(STATS_FILENAME), doc, 1);
     xmlFreeDoc(doc);
 }
 
